/**
 * 
 */
package commandingAgents;

import java.util.List;

import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.Parameters;
import repast.simphony.query.space.grid.GridCell;
import repast.simphony.query.space.grid.GridCellNgh;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.util.SimUtilities;

/**
 * @author Woo-Seop Yun
 *
 */
public class CompanyCommander {
	
	private ContinuousSpace<Object> space;
	private Grid<Object> grid;
	private String status;
	private String statusA;
	private String statusB;
	private String statusS;
	private boolean isStatusChange;
	private boolean isObstacle;
	private RedSoldier reportEnemyResult = null;
	Parameters params = RunEnvironment.getInstance().getParameters();
	private int DMTime = (Integer)params.getValue("coDMTime");
	private int DMTimeRand;
	private int DMCountA = 0;
	private int DMCountB = 0;
	private int DMCountC = 0;
	private int OrderPlTime = (Integer)params.getValue("coOrderPlTime");
	private int OrderPlTimeRand;
	private int OrderMoTime = (Integer)params.getValue("coOrderMoTime");
	private int OrderMoTimeRand;
	private int OrderMoCount = 0;
	private int OrderPlCountA = 0;
	private int OrderPlCountB = 0;
	private int OrderPlCountS = 0;
	private boolean mortarStatus = false;

	public CompanyCommander(ContinuousSpace<Object> space, Grid<Object> grid, String status, String statusA, String statusB, String statusS ){
		
		this.space = space;
		this.grid = grid;
		this.status = status;
		this.statusA = statusA;	// Status of Assault platoon
		this.statusB = statusB;	// Status of Breach platoon
		this.statusS = statusS;	// Status of Support platoon

	}
	
	@ScheduledMethod(start = 1, interval = 1, priority = -1)
	public void step(){

		DMTimeRand = RandomHelper.createPoisson(DMTime).nextInt();
		
		if(reportEnemyResult != null)
			mortarStatus = true;
			
			if(status == "Moving"){
				
				if(DMCountA == 0)
					isStatusChange = statusChange();
				
				if(isStatusChange){
					DMCountA++;
					if(DMCountA >= DMTimeRand){
						space.moveTo(this, 13.0, 47.0);
						NdPoint firstPoint = space.getLocation(this);
						grid.moveTo(this, (int)firstPoint.getX(), (int)firstPoint.getY());
						statusB = "Fighting";
						statusS = "Fighting";
						status = "Fighting";
						DMCountA = 0;
					}
				}
			}
			else if(status == "Fighting"){
				
				if(DMCountB == 0)
					isObstacle = isObstacle();
				
				if(!isObstacle){
					DMCountB++;
					if(DMCountB >= DMTimeRand){
						space.moveTo(this, 38.0, 47.0);
						NdPoint secondPoint = space.getLocation(this);
						grid.moveTo(this, (int)secondPoint.getX(), (int)secondPoint.getY());
						statusA = "Fighting";
						statusB = "Sweeping";
						statusS = "Sweeping";
						status = "Sweeping";
						DMCountB=0;
					}
					
				}
			}
			else if(status == "Sweeping"){
				
				if(DMCountC == 0)
					isStatusChange = statusChange();
				
				if(isStatusChange){
					DMCountC++;
					if(DMCountC >= DMTimeRand){
						statusA = "Sweeping";
						DMCountC=0;
					}
				}
			}
		}
	// }
	
	public boolean statusChange(){
		
		GridPoint pt = grid.getLocation(this);
		
		GridCellNgh<PlatoonLeader> nghCreator = new GridCellNgh<PlatoonLeader>(grid, pt, PlatoonLeader.class, 0, 0);
		List<GridCell<PlatoonLeader>> gridCells = nghCreator.getNeighborhood(true);
		SimUtilities.shuffle(gridCells, RandomHelper.getUniform());
		for(GridCell<PlatoonLeader> cell : gridCells){
			if(cell.size() > 0){
				return true;
			}
		}
		return false;
	}
	
	public boolean isObstacle(){
		
		GridPoint pt = grid.getLocation(this);
		
		GridCellNgh<Obstacle> nghCreator = new GridCellNgh<Obstacle>(grid, pt, Obstacle.class, 0, 0);
		List<GridCell<Obstacle>> gridCells = nghCreator.getNeighborhood(true);
		SimUtilities.shuffle(gridCells, RandomHelper.getUniform());
		for(GridCell<Obstacle> cell : gridCells){
			if(cell.size() > 0){
				return true;

			}
		}
		return false;
	}
	
	public String getStatusA(){
		if(OrderPlCountA == 0){
			OrderPlCountA = 1;
			return statusA;
		}
		OrderPlTimeRand = RandomHelper.createPoisson(OrderPlTime).nextInt();
		OrderPlCountA++;
		if(OrderPlCountA >= OrderPlTimeRand){
			OrderPlCountA = 1;
			return statusA;
		}
		return null;
	}
	public String getStatusB(){
		if(OrderPlCountB == 0){
			OrderPlCountB = 1;
			return statusB;
		}
		OrderPlTimeRand = RandomHelper.createPoisson(OrderPlTime).nextInt();
		OrderPlCountB++;
		if(OrderPlCountB >= OrderPlTimeRand){
			OrderPlCountB = 1;
			return statusB;
		}
		return null;
	}
	public String getStatusS(){
		if(OrderPlCountS == 0){
			OrderPlCountS = 1;
			return statusS;
		}
		OrderPlTimeRand = RandomHelper.createPoisson(OrderPlTime).nextInt();
		OrderPlCountS++;
		if(OrderPlCountS >= OrderPlTimeRand){
			OrderPlCountS = 1;
			return statusS;
		}
		return null;
	}
	
	public void reportEnemy(RedSoldier redsoldier){
		reportEnemyResult = redsoldier;
	}
	
	public RedSoldier getEnemy(){
		
		if(OrderMoCount == 0 && mortarStatus){
			OrderMoCount = 1;
			mortarStatus = false;
			return reportEnemyResult;
		}
		OrderMoTimeRand = RandomHelper.createPoisson(OrderMoTime).nextInt();
		OrderMoCount++;
		if(OrderMoCount >= OrderMoTimeRand){
			OrderMoCount = 1;
			mortarStatus = false;
			return reportEnemyResult;
		}
		return null;
	}
	
	public boolean getMortarStatus(){
		
		return mortarStatus;
		
	}
	
}
