/**
 * 
 */
package commandingAgents;

import java.util.ArrayList;
import java.util.List;

import commandingAgents.RedSoldier;
import repast.simphony.context.Context;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.query.space.grid.GridCell;
import repast.simphony.query.space.grid.GridCellNgh;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.util.ContextUtils;
import repast.simphony.util.SimUtilities;

/**
 * @author Woo-Seop Yun
 *
 */
public class Mortar {
	private ContinuousSpace<Object> space;
	private Grid<Object> grid;
	private int probKilling = 13;
	private CompanyCommander co = null;
	private RedSoldier target = null;
	private boolean firing = false;
	
	public Mortar(ContinuousSpace<Object> space, Grid<Object> grid){
		this.space = space;
		this.grid = grid;
	}
	
	@ScheduledMethod(start = 1, interval = 1, priority = 1)
	public void step(){
		if(co == null){
			for(Object obj : grid.getObjectsAt(15, 47)){
				if(obj instanceof CompanyCommander)
					this.co = (CompanyCommander) obj;
			}
		}
		
		if(!firing)
			target = co.getEnemy();
		
		
		if(target != null){
			firing = true;
			fire(target);
		}
		
	}

	
	public void fire(RedSoldier senseEnemyResult){
		
		Context<Object> context = ContextUtils.getContext(senseEnemyResult);
		
		GridPoint pt = grid.getLocation(senseEnemyResult);
		
		// Mortar fire to red soldiers within 4 grids (200 m)
		GridCellNgh<RedSoldier> nghCreator = new GridCellNgh<RedSoldier>(grid, pt, RedSoldier.class, 4, 4);
		List<GridCell<RedSoldier>> gridCells = nghCreator.getNeighborhood(true);
		SimUtilities.shuffle(gridCells, RandomHelper.getUniform());
		
		for(GridCell<RedSoldier> cell : gridCells){
			if(cell.size() > 0){

				List<RedSoldier> redsoldiers = new ArrayList<RedSoldier>();
				for(RedSoldier obj : cell.items()){
					redsoldiers.add(obj);
				}
				
				for(int i = 0 ; i < redsoldiers.size() ; i++ ){
					
					int randNumber = RandomHelper.nextIntFromTo(0, 100);
					
					RedSoldier redsoldier = redsoldiers.get(i);
					
					Network<Object> net = (Network<Object>) context.getProjection("mortar fire");
					net.addEdge(this, redsoldier);
					
					if(randNumber <= probKilling){
						context.remove(redsoldier);
						redsoldier = null;
					}	
				}
			}
		}
		firing = false;	
	}
}
