/**
 * 
 */
package commandingAgents;

import commandingAgents.Soldier;

import java.util.List;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.Parameters;
import repast.simphony.query.space.grid.GridCell;
import repast.simphony.query.space.grid.GridCellNgh;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.util.ContextUtils;
import repast.simphony.util.SimUtilities;

/**
 * @author Woo-Seop	Yun
 *
 */
public class Obstacle {
	
	private ContinuousSpace<Object> space;
	private Grid<Object> grid;
	private boolean status = true;	// true = Normal, false = Being breached 
	private boolean senseResult = false;
	Parameters params = RunEnvironment.getInstance().getParameters();
	private int breachDelayTime = (Integer)params.getValue("breachDelayTime");	// Time delay for breaching : 288 seconds with 2 or more blue soldiers, double times for a blue soldier alone
	private int breachCount = 0;	// For time advance of breaching
	
	public Obstacle(ContinuousSpace<Object> space, Grid<Object> grid){
		this.space = space;
		this.grid = grid;
	}
	
	public boolean getStatus(){
		return status;
	}
	
	@ScheduledMethod(start = 1, interval = 1, priority = -1)
	public void step(){
		
		if(senseResult){
			breach();
			return;
		}
		senseResult = sense();
	}
	
	public boolean sense(){
		GridPoint pt = grid.getLocation(this);
		
		GridCellNgh<Soldier> nghCreator = new GridCellNgh<Soldier>(grid, pt, Soldier.class, 0, 0);
		List<GridCell<Soldier>> gridCells = nghCreator.getNeighborhood(true);
		SimUtilities.shuffle(gridCells, RandomHelper.getUniform());
		for(GridCell<Soldier> cell : gridCells){
			if(cell.size() > 0){
				status = false;
				return true;
			}
		}
		status = true;
		return false;
	}
	
	public void breach(){
		
		breachCount++;
		
		if(breachCount == breachDelayTime){
			Context<Object> context = ContextUtils.getContext(this);
			context.remove(this);
		}
	}

}
