/**
 * 
 */
package commandingAgents;

import java.util.ArrayList;
import java.util.List;

import cern.jet.random.Poisson;
import commandingAgents.RedSoldier;



import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.engine.watcher.Watch;
import repast.simphony.engine.watcher.WatcherTriggerSchedule;
import repast.simphony.parameter.Parameters;
import repast.simphony.query.space.grid.GridCell;
import repast.simphony.query.space.grid.GridCellNgh;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.util.SimUtilities;

/**
 * @author Woo-Seop Yun
 *
 */
public class PlatoonLeader {

	private ContinuousSpace<Object> space;
	private Grid<Object> grid;
	private String mission;
	private GridPoint RP;	// Release Point
	private GridPoint midObjective;	// Intermediate Objective
	private String status = "Moving";	// "Moving", "Fighting", "Sweeping"
	private CompanyCommander co = null;
	private RedSoldier redsoldier = null;
	Parameters params = RunEnvironment.getInstance().getParameters();
	private int DMTime = (Integer)params.getValue("plDMTime");
	private int DMTimeRand;
	private int DMCount = 0;
	private int OrderTime = (Integer)params.getValue("plOrderTime");
	private int OrderTimeRand;
	private int OrderCount = 0;
	private String tempStatus = null;
	
	public PlatoonLeader(ContinuousSpace<Object> space, Grid<Object> grid, String mission, GridPoint RP, GridPoint midObjective){
		
		this.space = space;
		this.grid = grid;
		this.mission = mission;
		this.RP = RP;
		this.midObjective = midObjective;
	}
	
	@ScheduledMethod(start = 1, interval = 1, priority = 1)
	public void status(){
		if(co == null){
			for(Object obj : grid.getObjectsAt(15, 47)){
				if(obj instanceof CompanyCommander)
					this.co = (CompanyCommander) obj;
			}
		}
		
		if(mission == "Assault"){
			tempStatus = co.getStatusA();
			if(tempStatus != null)
				status = tempStatus;
		}
		else if(mission == "Breach"){
			tempStatus = co.getStatusB();
			if(tempStatus != null)
				status = tempStatus;
		}
		else{
			tempStatus = co.getStatusS();
			if(tempStatus != null)
				status = tempStatus;
		}
		redsoldier = senseEnemy();
		
		co.reportEnemy(redsoldier);

	}
	
	@Watch(watcheeClassName = "commandingAgents.Soldier", watcheeFieldNames = "moved", query = "within_moore 1", whenToTrigger = WatcherTriggerSchedule.IMMEDIATE)
	public void step(){
		
		if(status == "Moving"){
			
			space.moveTo(this, (double)midObjective.getX(), (double)midObjective.getY());
			NdPoint myPoint = space.getLocation(this);
			grid.moveTo(this, (int)myPoint.getX(), (int)myPoint.getY());
		}
		else if(status == "Fighting"){
			if(mission == "Assault"){
				space.moveTo(this, 38.0, 47.0);
				NdPoint myPoint = space.getLocation(this);
				grid.moveTo(this, (int)myPoint.getX(), (int)myPoint.getY());
			}
		}
		
	}
	
	public RedSoldier senseEnemy(){
		
		GridPoint pt = grid.getLocation(this);
		
		// Sensing a red soldier to fire within 10 grids (500 m)
		GridCellNgh<RedSoldier> nghCreator = new GridCellNgh<RedSoldier>(grid, pt, RedSoldier.class, 10, 10);
		List<GridCell<RedSoldier>> gridCells = nghCreator.getNeighborhood(true);
		SimUtilities.shuffle(gridCells, RandomHelper.getUniform());
		
		for(GridCell<RedSoldier> cell : gridCells){
			if(cell.size() > 0){
				DMTimeRand = RandomHelper.createPoisson(DMTime).nextInt();
				DMCount++;
				if(DMCount >= DMTimeRand){
					List<RedSoldier> redsoldiers = new ArrayList<RedSoldier>();
					for(RedSoldier obj : cell.items()){
						redsoldiers.add(obj);
					}
					int index = RandomHelper.nextIntFromTo(0, redsoldiers.size()-1);
					RedSoldier obj = redsoldiers.get(index);
					DMCount = 0;
					return obj;
				}
			}
		}
		return null;
	}
	
	
	public String getMission(){
		return mission;
	}
	public String getStatus(){
		if(OrderCount == 0)
			return status;
		OrderTimeRand = RandomHelper.createPoisson(OrderTime).nextInt();
		OrderCount++;
		if(OrderCount >= OrderTimeRand){
			OrderCount = 1;
			return status;
		}
		return null;
	}
	public GridPoint getRP(){
		return RP;
	}
	public GridPoint getMidObjective(){
		return midObjective;
	}
	
}