/**
 * 
 */
package commandingAgents;

import commandingAgents.Soldier;
import commandingAgents.Terrain;

import java.util.ArrayList;
import java.util.List;

import repast.simphony.context.Context;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.query.space.grid.GridCell;
import repast.simphony.query.space.grid.GridCellNgh;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.SpatialMath;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.util.ContextUtils;
import repast.simphony.util.SimUtilities;

/**
 * @author Woo-Seop Yun
 *
 */
public class RedSoldier {
	
	private ContinuousSpace<Object> space;
	private Grid<Object> grid;
	private Soldier senseEnemyResult = null;
	private String enemyStatus = "Close";
	private int probKillingCoveredClose = 18;
	private int probKillingCoveredFar = 16;
	private int probKillingOpenClose = 32;
	private int probKillingOpenFar = 28;
	
	public RedSoldier(ContinuousSpace<Object> space, Grid<Object> grid){
		this.space = space;
		this.grid = grid;
	}
	
	@ScheduledMethod(start = 1, interval = 1, priority = 1)
	public void step(){
		
		senseEnemyResult = sense();
		fire(senseEnemyResult);
	}
	
	public Soldier sense(){
		
		GridPoint pt = grid.getLocation(this);
		
		// Sensing a blue soldier to fire within 2 grids (100 m)
		GridCellNgh<Soldier> nghCreator1 = new GridCellNgh<Soldier>(grid, pt, Soldier.class, 2, 2);
		List<GridCell<Soldier>> gridCells1 = nghCreator1.getNeighborhood(true);
		SimUtilities.shuffle(gridCells1, RandomHelper.getUniform());
		
		for(GridCell<Soldier> cell1 : gridCells1){
			if(cell1.size() > 0){
				enemyStatus = "Close";
				List<Soldier> soldiers = new ArrayList<Soldier>();
				for(Soldier obj : cell1.items()){
					soldiers.add(obj);
				}
				int index = RandomHelper.nextIntFromTo(0, soldiers.size()-1);
				Soldier obj = soldiers.get(index);
				return obj;
			}
		}	
		
		// Sensing a blue soldiers to fire within 4 grids (200 m)
		GridCellNgh<Soldier> nghCreator2 = new GridCellNgh<Soldier>(grid, pt, Soldier.class, 4, 4);
		List<GridCell<Soldier>> gridCells2 = nghCreator2.getNeighborhood(true);
		SimUtilities.shuffle(gridCells2, RandomHelper.getUniform());
		
		for(GridCell<Soldier> cell2 : gridCells2){
			if(cell2.size() > 0){
				
				enemyStatus = "Far";
				List<Soldier> soldiers = new ArrayList<Soldier>();
				for(Soldier obj : cell2.items()){
					soldiers.add(obj);
				}
				int index = RandomHelper.nextIntFromTo(0, soldiers.size()-1);
				Soldier obj = soldiers.get(index);
				
				return obj;
			}
		}
		return null;
	}
	
	public void fire(Soldier senseEnemyResult){
		
		if(senseEnemyResult == null){
			return;
		}
	
		GridPoint pt = grid.getLocation(senseEnemyResult);
		Terrain terrain = null;
		for(Object obj : grid.getObjectsAt(pt.getX(), pt.getY())){
			if(obj instanceof Terrain){
				terrain = (Terrain) obj;
			}
		}
		
		int mountain = terrain.getMountain();
		
		Context<Object> context = ContextUtils.getContext(senseEnemyResult);
		int randNumber = RandomHelper.nextIntFromTo(0, 100);
		
		Network<Object> net = (Network<Object>) context.getProjection("red fire");
		net.addEdge(this, senseEnemyResult);
		

		if(enemyStatus == "Close"){
			if(mountain > 0){
				if(randNumber <= probKillingCoveredClose){
					context.remove(senseEnemyResult);
					senseEnemyResult = null;
				}
			}
			else{
				if(randNumber <= probKillingOpenClose){
					context.remove(senseEnemyResult);
					senseEnemyResult = null;
				}
			}
		}
		else{
			if(mountain > 0){
				if(randNumber <= probKillingCoveredFar){
					context.remove(senseEnemyResult);
					senseEnemyResult = null;
				}
			}
			else{
				if(randNumber <= probKillingOpenFar){
					context.remove(senseEnemyResult);
					senseEnemyResult = null;
				}
			}
			
		}
	}
}