/**
 * 
 */
package commandingAgents;

import commandingAgents.Obstacle;
import commandingAgents.RedSoldier;
import commandingAgents.Terrain;
import commandingAgents.PlatoonLeader;

import java.util.ArrayList;
import java.util.List;

import repast.simphony.context.Context;
import repast.simphony.engine.environment.RunEnvironment;
import repast.simphony.engine.schedule.ScheduledMethod;
import repast.simphony.parameter.Parameters;
import repast.simphony.query.space.grid.GridCell;
import repast.simphony.query.space.grid.GridCellNgh;
import repast.simphony.random.RandomHelper;
import repast.simphony.space.SpatialMath;
import repast.simphony.space.continuous.ContinuousSpace;
import repast.simphony.space.continuous.NdPoint;
import repast.simphony.space.graph.Network;
import repast.simphony.space.grid.Grid;
import repast.simphony.space.grid.GridPoint;
import repast.simphony.util.ContextUtils;
import repast.simphony.util.SimUtilities;

/**
 * @author Woo-Seop Yun
 *
 */
public class Soldier {
	
	private ContinuousSpace<Object> space;
	private Grid<Object> grid;
	private GridPoint RP;
	private GridPoint midObjective;
	private GridPoint objective;
	private PlatoonLeader pl = null;
	private int id;
	private RedSoldier senseEnemyResult = null;
	private String enemyStatus = "Close";
	Parameters params = RunEnvironment.getInstance().getParameters();
	private int probKillingCoveredClose = (Integer)params.getValue("probKillingCoveredClose");
	private int probKillingCoveredFar = (Integer)params.getValue("probKillingCoveredFar");
	private int probKillingOpenClose = (Integer)params.getValue("probKillingOpenClose");
	private int probKillingOpenFar = (Integer)params.getValue("probKillingOpenFar");
	private String plStatus = null;
	private String mission = null;
	private String status = "Moving";	// "Moving", "Firing", "Breaching"
	private int moveDelayTime;
	private int moveCount = 0;
	private boolean moved;
	
	public Soldier(ContinuousSpace<Object> space, Grid<Object> grid, GridPoint objective, int id){
		this.space = space;
		this.grid = grid;
		this.objective = objective;
		this.id = id;
	}
	
	@ScheduledMethod(start = 1, interval = 1, priority = 1)
	public void step(){

		if(pl == null){
			if(id<30){
				for(Object obj : grid.getObjectsAt(3, 47)){
					if(obj instanceof PlatoonLeader){
						this.pl = (PlatoonLeader) obj;
						this.RP = pl.getRP();
						this.midObjective = pl.getMidObjective();
						this.mission = pl.getMission();
					}
				}
			}
			else if(id<60){
				for(Object obj : grid.getObjectsAt(5, 48)){
					if(obj instanceof PlatoonLeader){
						this.pl = (PlatoonLeader) obj;
						this.RP = pl.getRP();
						this.midObjective = pl.getMidObjective();
						this.mission = pl.getMission();
					}
				}
			}
			else{
				for(Object obj : grid.getObjectsAt(25, 28)){
					if(obj instanceof PlatoonLeader){
						this.pl = (PlatoonLeader) obj;
						this.RP = pl.getRP();
						this.midObjective = pl.getMidObjective();
						this.mission = pl.getMission();
					}
				}
			}
		}
		
		String tempStatus = pl.getStatus();
		if(tempStatus != null)
			plStatus = tempStatus;
		
		if(status != "Breaching"){
			senseBreach();
		}
		if(status == "Breaching"){
			senseBreach();
			if(status == "Breaching")
				return;
		}
		
		senseEnemyResult = senseEnemy();
		if(status == "Firing"){
			fire(senseEnemyResult);
		}
		else{
			if(plStatus == "Moving"){
				if(mission == "Support"){
					GridPoint RPS = new GridPoint(RP.getX()+(int)((id*(-1)%60)/2), RP.getY());
					move(RPS);
				}
				move(RP);
			}
			else if(plStatus == "Fighting"){
				if(mission == "Support"){
					GridPoint midObjectiveS = new GridPoint(midObjective.getX()+(int)((id*(-1)%60)/2), midObjective.getY());
					move(midObjectiveS);
				}
				move(midObjective);
			}
			else if(plStatus == "Sweeping"){
				move(objective);
				
			}
		}
	}
		
	
	public void senseBreach(){
		GridPoint pt = grid.getLocation(this);
		
		GridCellNgh<Obstacle> nghCreator = new GridCellNgh<Obstacle>(grid, pt, Obstacle.class, 0, 0);
		List<GridCell<Obstacle>> gridCells = nghCreator.getNeighborhood(true);
		SimUtilities.shuffle(gridCells, RandomHelper.getUniform());
		for(GridCell<Obstacle> cell : gridCells){
			if(cell.size() > 0){
				if(status == "Breaching"){
					return;
				}
				List<Obstacle> obstacles = new ArrayList<Obstacle>();
				for(Obstacle obj : cell.items()){
					obstacles.add(obj);
				}
				int index = RandomHelper.nextIntFromTo(0, obstacles.size()-1);
				Obstacle obj = obstacles.get(index);
				if(obj.getStatus()){
					status = "Breaching";
				}
				else{
					status = "Firing";
				}
			}
			else{
				status = "Moving";
			}
		}
	}
	
public RedSoldier senseEnemy(){
		
		GridPoint pt = grid.getLocation(this);
		
		// Sensing a red soldier to fire within 2 grids (100 m)
		GridCellNgh<RedSoldier> nghCreator1 = new GridCellNgh<RedSoldier>(grid, pt, RedSoldier.class, 2, 2);
		List<GridCell<RedSoldier>> gridCells1 = nghCreator1.getNeighborhood(true);
		SimUtilities.shuffle(gridCells1, RandomHelper.getUniform());
		
		for(GridCell<RedSoldier> cell1 : gridCells1){
			if(cell1.size() > 0){
				status = "Firing";
				enemyStatus = "Close";
				List<RedSoldier> redsoldiers = new ArrayList<RedSoldier>();
				for(RedSoldier obj : cell1.items()){
					redsoldiers.add(obj);
				}
				int index = RandomHelper.nextIntFromTo(0, redsoldiers.size()-1);
				RedSoldier obj = redsoldiers.get(index);
				return obj;
			}
		}	
		
		// Sensing a blue soldiers to fire within 4 grids (200 m)
		GridCellNgh<RedSoldier> nghCreator2 = new GridCellNgh<RedSoldier>(grid, pt, RedSoldier.class, 4, 4);
		List<GridCell<RedSoldier>> gridCells2 = nghCreator2.getNeighborhood(true);
		SimUtilities.shuffle(gridCells2, RandomHelper.getUniform());
		
		for(GridCell<RedSoldier> cell2 : gridCells2){
			if(cell2.size() > 0){
				status = "Firing";
				enemyStatus = "Far";
				List<RedSoldier> redsoldiers = new ArrayList<RedSoldier>();
				for(RedSoldier obj : cell2.items()){
					redsoldiers.add(obj);
				}
				int index = RandomHelper.nextIntFromTo(0, redsoldiers.size()-1);
				RedSoldier obj = redsoldiers.get(index);
				
				return obj;
			}
		}
		return null;
	}
	
	public void fire(RedSoldier senseEnemyResult){
		
		if(senseEnemyResult == null){
			status = "Moving";
			return;
		}
	
		GridPoint pt = grid.getLocation(senseEnemyResult);
		Terrain terrain = null;
		for(Object obj : grid.getObjectsAt(pt.getX(), pt.getY())){
			if(obj instanceof Terrain){
				terrain = (Terrain) obj;
			}
		}
		
		int mountain = terrain.getMountain();
		
		Context<Object> context = ContextUtils.getContext(senseEnemyResult);
		int randNumber = RandomHelper.nextIntFromTo(0, 100);
		
		Network<Object> net = (Network<Object>) context.getProjection("blue fire");
		net.addEdge(this, senseEnemyResult);
		
	
		if(enemyStatus == "Close"){
			if(mountain > 0){
				if(randNumber <= probKillingCoveredClose){
					context.remove(senseEnemyResult);
					status = "Moving";
					senseEnemyResult = null;
				}
			}
			else{
				if(randNumber <= probKillingOpenClose){
					context.remove(senseEnemyResult);
					status = "Moving";
					senseEnemyResult = null;
				}
			}
		}
		else{
			if(mountain > 0){
				if(randNumber <= probKillingCoveredFar){
					context.remove(senseEnemyResult);
					status = "Moving";
					senseEnemyResult = null;
				}
			}
			else{
				if(randNumber <= probKillingOpenFar){
					context.remove(senseEnemyResult);
					status = "Moving";
					senseEnemyResult = null;
				}
			}
			
		}
	}
	
	public void move(GridPoint headTo){
		
		GridPoint pt = grid.getLocation(this);
		
		int distanceX = pt.getX() - headTo.getX();
		int distanceY = pt.getY() - headTo.getY();
		
		if(Math.abs(distanceX)>1||Math.abs(distanceY)>1){
			Terrain terrain = null;
			for(Object obj : grid.getObjectsAt(pt.getX(), pt.getY())){
				if(obj instanceof Terrain){
					terrain = (Terrain) obj;
				}
			}
			int mountain = terrain.getMountain();
			if(mountain==2){
				moveDelayTime = 100;
			}
			else if(mountain==1){
				moveDelayTime = 67;
			}
			else{
				moveDelayTime = 50;
			}
			
			moveCount++;
			
			if(moveCount >= moveDelayTime){
				NdPoint myPoint = space.getLocation(this);
				NdPoint otherPoint = new NdPoint(headTo.getX(), headTo.getY());
				double angle = SpatialMath.calcAngleFor2DMovement(space, myPoint, otherPoint);
				space.moveByVector(this, 1, angle, 0);
				myPoint = space.getLocation(this);
				grid.moveTo(this, (int)myPoint.getX(), (int)myPoint.getY());
				moved = true;
				moveCount = 0;
			}
		}
	}
}
